 /*
 * for the Matrox Marvel G200 and Rainbow Runner-G series
 *
 * This module is an interface to the ZR3606 jpeg codec chip.
 *
 * Copyright (C) 1999  Mike Bernson <mike@mlb.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * This code is based on Rainer Johanni buz driver. <Rainer@Johanni.de>
 *
 *****************************************************************************
 */
#ifndef __KERNEL__
#define __KERNEL__
#endif
#ifndef MODULE
#define MODULE
#endif

#include <linux/module.h>
#include <linux/delay.h>
#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/major.h>
#include <linux/malloc.h>
#include <linux/mm.h>
#include <linux/pci.h>
#include <linux/signal.h>
#include <asm/io.h>
#include <asm/pgtable.h>
#include <asm/page.h>
#include <linux/sched.h>
#include <asm/segment.h>
#include <linux/types.h>
#include <linux/wrapper.h>

#include <linux/videodev.h>
#include <linux/version.h>
#include <asm/uaccess.h>

#if	LINUX_VERSION_CODE >= 0x020300
#include <linux/i2c-old.h>
#else
#include <linux/i2c.h>
#endif

#include "zr36060.h"
#include "tvnorm.h"

#define dprintk     if (debug) printk

static int debug = 1; /* insmod parameter */

#if LINUX_VERSION_CODE >= 0x020100
MODULE_PARM(debug,"i");
#endif

#define ZR_NAME      "zr36060"   /* name of the driver */

#define ZR_ERR       KERN_ERR     ZR_NAME
#define ZR_DEBUG     KERN_INFO    ZR_NAME
#define ZR_INFO      KERN_INFO    ZR_NAME
#define ZR_WARNING   KERN_WARNING ZR_NAME


#define	MAX_KMALLOC_MEM	(32 * 1024)

static void jpg_fbuffer_free(struct zr36060_dev *zr);

/*
 *   Allocate the MJPEG grab buffers.
 *
 *   If the requested buffer size is smaller than MAX_KMALLOC_MEM,
 *   kmalloc is used to request a physically contiguous area,
 *   else we allocate the memory in framgents with get_free_page.
 *
 *   If a Natoma chipset is present and this is a revision 1 zr36057,
 *   each MJPEG buffer needs to be physically contiguous.
 *   (RJ: This statement is from Dave Perks' original driver,
 *   I could never check it because I have a zr36067)
 *   The driver cares about this because it reduces the buffer
 *   size to MAX_KMALLOC_MEM in that case (which forces contiguous allocation).
 *
 *   RJ: The contents grab buffers needs never be accessed in the driver.
 *       Therefore there is no need to allocate them with vmalloc in order
 *       to get a contiguous virtual memory space.
 *       I don't understand why many other drivers first allocate them with
 *       vmalloc (which uses internally also get_free_page, but delivers you
 *       virtual addresses) and then again have to make a lot of efforts
 *       to get the physical address.
 *
 */

static int jpg_fbuffer_alloc(struct zr36060_dev * zr)
{
   int i, j, off, alloc_contig;
   unsigned long mem;

   /* Decide if we should alloc contiguous or fragmented memory */
   /* This has to be identical in jpg_fbuffer_alloc and jpg_fbuffer_free */

   alloc_contig = (zr->jpg_bufsize < MAX_KMALLOC_MEM);

   for (i = 0; i < zr->jpg_nbufs; i++)
   {
      if(zr->jpg_gbuf[i].frag_tab)
         printk(ZR_WARNING ": jpg_fbuffer_alloc: buffer %d allready allocated ???\n",i);

      /* Allocate fragment table for this buffer */
      mem = get_free_page(GFP_KERNEL);
      if (mem==0) 
      {
         printk(ZR_ERR ": jpg_fbuffer_alloc: get_free_page (frag_tab) failed for buffer %d\n",i);
         jpg_fbuffer_free(zr);
         return -ENOBUFS;
      }
      memset((void*)mem,0,PAGE_SIZE);
      zr->jpg_gbuf[i].frag_tab     = (u32*) mem;
      zr->jpg_gbuf[i].frag_tab_bus = virt_to_bus((void*)mem);

      /* Allocate table virt address/size for this buffer */
      mem = get_free_page(GFP_KERNEL);
      if (mem==0) 
      {
         printk(ZR_ERR ": jpg_fbuffer_alloc: get_free_page (frag_tab) failed for buffer %d\n",i);
         jpg_fbuffer_free(zr);
         return -ENOBUFS;
      }
      memset((void*)mem,0,PAGE_SIZE);
      zr->jpg_gbuf[i].virt_tab     = (u32*) mem;

      if(alloc_contig)
      {
         mem = (unsigned long)kmalloc(zr->jpg_bufsize,GFP_KERNEL);
         if(mem==0)
         {
            printk(ZR_ERR ": jpg_fbuffer_alloc: kmalloc failed for buffer %d\n",i);
            jpg_fbuffer_free(zr);
            return -ENOBUFS;
         }
	 zr->jpg_gbuf[i].virt_tab[0] = mem;
	 zr->jpg_gbuf[1].virt_tab[1] = zr->jpg_bufsize;

         zr->jpg_gbuf[i].frag_tab[0] = virt_to_bus((void*)mem);
         zr->jpg_gbuf[i].frag_tab[1] = ((zr->jpg_bufsize/4)<<1)|1;

         for(off=0;off<zr->jpg_bufsize;off+=PAGE_SIZE) mem_map_reserve(MAP_NR(mem+off));
      }
      else
      {
         /* jpg_bufsize is allreay page aligned */
         for (j=0; j<zr->jpg_bufsize/PAGE_SIZE; j++)
         {
            mem = get_free_page(GFP_KERNEL);

            if(mem==0)
            {
               printk(ZR_ERR ": jpg_fbuffer_alloc: get_free_page failed for buffer %d\n",i);
               jpg_fbuffer_free(zr);
               return -ENOBUFS;
            }

	    zr->jpg_gbuf[i].virt_tab[2*j  ] = mem;
            zr->jpg_gbuf[i].virt_tab[2*j+1] = PAGE_SIZE;

            zr->jpg_gbuf[i].frag_tab[2*j  ] = virt_to_bus((void*)mem);
            zr->jpg_gbuf[i].frag_tab[2*j+1] = (PAGE_SIZE/4)<<1;

            mem_map_reserve(MAP_NR(mem));
         }

         zr->jpg_gbuf[i].frag_tab[2*j-1] |= 1;
         zr->jpg_gbuf[i].virt_tab[2*j-1] |= 1;
      }
   }

   printk("jpg_fbuffer_alloc: %d KB allocated\n",
                (zr->jpg_nbufs*zr->jpg_bufsize)>>10);
   zr->jpg_buffers_allocated = 1;
   return 0;
}

/* free the MJPEG grab buffers */
static void jpg_fbuffer_free(struct zr36060_dev * zr)
{
   int i, j, off, alloc_contig;
   unsigned char *mem;

   if (zr->jpg_buffers_allocated == 0)
	return;

   /* Decide if we should alloc contiguous or fragmented memory */
   /* This has to be identical in jpg_fbuffer_alloc and jpg_fbuffer_free */

   alloc_contig = (zr->jpg_bufsize < MAX_KMALLOC_MEM);

   for (i = 0; i < zr->jpg_nbufs; i++)
   {
      if(!zr->jpg_gbuf[i].frag_tab) continue;

      if(alloc_contig)
      {
         if(zr->jpg_gbuf[i].frag_tab[0])
         {
            mem = (unsigned char *) bus_to_virt(zr->jpg_gbuf[i].frag_tab[0]);
            for(off=0;off<zr->jpg_bufsize;off+=PAGE_SIZE) mem_map_unreserve(MAP_NR(mem+off));
            kfree((void *) mem);
            zr->jpg_gbuf[i].frag_tab[0] = 0;
            zr->jpg_gbuf[i].frag_tab[1] = 0;
            zr->jpg_gbuf[i].virt_tab[0] = 0;
            zr->jpg_gbuf[i].virt_tab[1] = 0;
         }
      }
      else
      {
         for(j=0; j<zr->jpg_bufsize/PAGE_SIZE; j++)
         {
            if(!zr->jpg_gbuf[i].frag_tab[2*j]) break;
            mem_map_unreserve(MAP_NR(bus_to_virt(zr->jpg_gbuf[i].frag_tab[2*j])));
            free_page((unsigned long)bus_to_virt(zr->jpg_gbuf[i].frag_tab[2*j]));
            zr->jpg_gbuf[i].frag_tab[2*j  ] = 0;
            zr->jpg_gbuf[i].frag_tab[2*j+1] = 0;
         }
      }

      free_page( (unsigned long)zr->jpg_gbuf[i].frag_tab );
      free_page( (unsigned long)zr->jpg_gbuf[i].virt_tab );

      zr->jpg_gbuf[i].frag_tab = NULL;
      zr->jpg_gbuf[i].virt_tab = NULL;
   }
   zr->jpg_buffers_allocated = 0;
}

int
zr36060_mmap_bufs(struct zr36060_dev *zr, unsigned long adr, unsigned long size)
{
	int	i;
	int	j;
	unsigned long start;
	unsigned long page;
	unsigned long todo;
	unsigned long pos;
	unsigned long fraglen;

	start = adr;

      /* Map the MJPEG buffers */

      if(!zr->jpg_buffers_allocated)
      {
         printk(ZR_ERR ": zr36060_mmap_bufs(MJPEG): buffers not yet allocated\n");
         return -ENOMEM;
      }

      if(size > zr->jpg_nbufs * zr->jpg_bufsize)
      {
         printk(ZR_ERR ": zr36060_mmap_bufs(MJPEG): Max size is %u - you wanted %lu\n",
                zr->jpg_nbufs * zr->jpg_bufsize, size);
         return -EINVAL;
      }

      if(size != zr->jpg_nbufs * zr->jpg_bufsize)
         printk(ZR_WARNING ": buz_mmap(MJPEG): Expected %u - you wanted %lu\n",
                zr->jpg_nbufs * zr->jpg_bufsize, size);

      for(i=0; i < zr->jpg_nbufs; i++)
      {
         for(j=0; j < zr->jpg_bufsize/PAGE_SIZE; j++)
         {
            fraglen = (zr->jpg_gbuf[i].frag_tab[2*j+1] & ~1) << 1;
            todo = size;
            if(todo>fraglen) todo = fraglen;
            pos = (unsigned long)zr->jpg_gbuf[i].frag_tab[2*j];
            page = virt_to_phys(bus_to_virt(pos)); /* should just be pos on i386 */
            if (remap_page_range(start, page, todo, PAGE_SHARED)) {
               printk(ZR_ERR ": buz_mmap(V4L): remap_page_range failed\n");
               return -EAGAIN;
            }
            size  -= todo;
            start += todo;
            if(size==0) break;
            if(zr->jpg_gbuf[i].frag_tab[2*j+1] & 1) break; /* was last fragment */
         }
         if(size==0) break;
      }
   return 0;
}

/* Check a buz_params struct for correctness, insert default params */
static int zr36060_check_params(struct zr36060_dev *zr, struct buz_params * params)
{
   int err = 0, err0 = 0;

   /* insert constant params */
   params->major_version = 0;
   params->minor_version = 0xdeadbeef;

   /* Check input and norm */

   if(params->input < 0 || params->input > zr->input_count)
   {
      printk(ZR_ERR "-%u CHECK PARAMS: input = %d, must be 0-%d\n",
		zr->id, params->input, zr->input_count);
      err++;
   }
   
   /* Deactivated in the moment */
   /*     if(params->norm != VIDEO_MODE_PAL && params->norm != VIDEO_MODE_NTSC) */
   /*       { */
   /*       printk(ZR_ERR " CHECK PARAMS: norm = %d, must be 0 (PAL) or 1 (NTSC)\n",params->norm); */
   /*       err++; */
   /*       } */

   /* Check decimation, set default values for decimation = 1, 2, 4 */

   switch (params->decimation)
   {
      case 1:

         params->HorDcm         = 1;
         params->VerDcm         = 1;
         params->TmpDcm         = 1;
         params->field_per_buff = 2;

         params->img_x          = 0;
         params->img_y          = 0;
         params->img_width      = 720;
         params->img_height     = zr->norms[params->norm].Ha/2;
         break;

      case 2:

         params->HorDcm         = 2;
         params->VerDcm         = 1;
         params->TmpDcm         = 2;
         params->field_per_buff = 1;

         params->img_x          = 8;
         params->img_y          = 0;
         params->img_width      = 704;
         params->img_height     = zr->norms[params->norm].Ha/2;
         break;

      case 4:

         params->HorDcm         = 4;
         params->VerDcm         = 2;
         params->TmpDcm         = 2;
         params->field_per_buff = 1;

         params->img_x          = 8;
         params->img_y          = 0;
         params->img_width      = 704;
         params->img_height     = zr->norms[params->norm].Ha/2;
         break;

      case 0:

         /* We have to check the data the user has set */

         if(params->HorDcm!=1 && params->HorDcm!=2 && params->HorDcm!=4) err0++;
         if(params->VerDcm!=1 && params->VerDcm!=2) err0++;
         if(params->TmpDcm!=1 && params->TmpDcm!=2) err0++;
         if(params->field_per_buff!=1 && params->field_per_buff!=2) err0++;

         if(params->img_x     <0) err0++;
         if(params->img_y     <0) err0++;
         if(params->img_width <0) err0++;
         if(params->img_height<0) err0++;
         if(params->img_x + params->img_width  > 720) err0++;
         if(params->img_y + params->img_height > zr->norms[params->norm].Ha/2) err0++;
         if(params->img_width  % (16*params->HorDcm) != 0 ) err0++;
         if(params->img_height % ( 8*params->VerDcm) != 0 ) err0++;

         if(err0)
         {
            printk(ZR_ERR "-%u: SET PARAMS: error in params for decimation = 0\n", zr->id);
            err++;
         }
         break;

      default:
         printk(ZR_ERR "-%u:  SET PARAMS: decimation = %d, must be 0, 1, 2 or 4\n",zr->id, params->decimation);
         err++;
         break;
   }

   if(params->quality > 100) params->quality = 100;
   if(params->quality <   5) params->quality =   5;

   if(params->APPn <  0) params->APPn =  0;
   if(params->APPn > 15) params->APPn = 15;
   if(params->APP_len <  0) params->APP_len =  0;
   if(params->APP_len > 60) params->APP_len = 60;
   if(params->COM_len <  0) params->COM_len =  0;
   if(params->COM_len > 60) params->COM_len = 60;

   if(err) return -EINVAL;

   return 0;

}

static int zr36060_default_params(struct zr36060_dev *zr)
{
   int i;

   /* re-initialize DMA ring stuff */  
   zr->jpg_que_head = 0;
   zr->jpg_dma_head = 0;
   zr->jpg_dma_tail = 0;
   zr->jpg_que_tail = 0;
   zr->jpg_seq_num  = 0;

   for(i=0; i < zr->jpg_nbufs; i++)
	zr->jpg_gbuf[i].state = BUZ_STATE_USER;

   zr->map_mjpeg_buffers= 0;

#if LINUX_VERSION_CODE >= 0x020300
   init_waitqueue_head(&btv->jpg_capq);
#endif

   /* Set necessary params and call buz_check_params to set the defaults */
   zr->params.decimation = 1;

   zr->params.quality    = 50; /* default compression factor 8 */
   zr->params.odd_even   = 1;

   zr->params.APPn    = 0;
   zr->params.APP_len = 0; /* No APPn marker */
   for(i=0;i<60;i++) 
	zr->params.APP_data[i] = 0;

   zr->params.COM_len = 0; /* No COM marker */
   for(i=0;i<60;i++) 
	zr->params.COM_data[i] = 0;

   zr->params.VFIFO_FB = 0;

   memset(zr->params.reserved,0,sizeof(zr->params.reserved));

   zr->params.jpeg_markers = JPEG_MARKER_DHT | JPEG_MARKER_DQT;

   i = zr36060_check_params(zr, &zr->params);
   if(i) printk(ZR_ERR "-%u: buz_open_init_params internal error\n", zr->id);

   return i;
}

/****************************************************************************
* linux kernel module api
****************************************************************************/
#ifdef MODULE
int init_module(void)
#else
int zr36060_init(void)
#endif
{

	return 0;
}

#ifdef MODULE
void cleanup_module(void)
{
}
#endif

static int
zr36060_load_memory(struct zr36060_dev *zr) 
{
	int	i;
	char	status;
	char	data[1];

	data[0] = 0x80;
	zr->codec_write_memory(zr, data, 0, 1);

	for(i=0; i < 1000; i++) {
		zr->codec_read_memory(zr, &status, 1, 1);

		if ((status & 0x80) == 0) {
			return 0;
		}
		udelay(10);
	}

	printk(ZR_ERR "-%u: load from memory did not complete status=0x%x\n",
		zr->id, status);
	return -1;
 }



static u8 SOF[19] = {
	0xff, 0xc0,		/* Start of frame marker */
	0x00, 0x11,		/* Length of this segment */
	0x08,			/* Precision */
	0x00, 0x78,		/* number of lines */
	0x00, 0xb0,		/* line length */
	3,			/* number of components */
	0, 0x21, 0,		/* Y component */
	1, 0x11, 1,		/* U component */
	2, 0x11, 1		/* V component */
};

static u8 SOS[14] = {
	0xff, 0xda,		/* start of Scan marker */
	0x00, 0x0c,		/* lenght of this segment */
	0x03,			/* number of components */
	0x00, 0x00,		/* Y component */
	0x01, 0x11,		/* U component */
	0x02, 0x11,		/* V component */
	0x00, 0x3f, 0x00	/* constant */
};

static u8 DRI[6] = {
	0xff, 0xdd,		/* define Restart Interval */
	0x00, 0x04,		/* length of this segment */
	0x00, 0x08,		/* lenght of restart interval in mcu units */
};

static u8 DQT[134] = {
	0xff, 0xdb,		/* DQT marker */
	0x00, 0x84,		/* DQT length */
	0x00,			/* table ID 0 */
	0x10, 0x0b, 0x0c, 0x0e, 0x0c, 0x0a, 0x10, 0x0e,
	0x0d, 0x0e, 0x12, 0x11, 0x10, 0x13, 0x18, 0x28,
	0x1a, 0x18, 0x16, 0x16, 0x18, 0x31, 0x23, 0x25,
	0x1d, 0x28, 0x3a, 0x33, 0x3d, 0x3c, 0x39, 0x33,
	0x38, 0x37, 0x40, 0x48, 0x5c, 0x4e, 0x40, 0x44,
	0x57, 0x45, 0x37, 0x38, 0x50, 0x6d, 0x51, 0x57,
	0x5f, 0x62, 0x67, 0x68, 0x67, 0x3e, 0x4d, 0x71,
	0x79, 0x70, 0x64, 0x78, 0x5c, 0x65, 0x67, 0x63,
	0x01,			/* table ID 1 */
	0x11, 0x12, 0x12, 0x18, 0x15, 0x18, 0x2f, 0x1a,
	0x1a, 0x2f, 0x63, 0x42, 0x38, 0x42, 0x63, 0x63,
	0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63,
	0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63,
	0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63,
	0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63,
	0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63,
	0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63
};

static u8 DHT[420] = {
      0xff, 0xc4,   /* DHT marker */
      0x01, 0xa2,   /* DHT length */
      0x00,      /* table class 0, ID 0 */
      0x00, 0x01, 0x05, 0x01, 0x01, 0x01, 0x01, 0x01, /* # codes of length 1..8 */
      0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* # codes of length 8..16 */
      0x00,                  /* values for codes of length 2 */
      0x01, 0x02, 0x03, 0x04, 0x05,         /* values for codes of length 3 */
      0x06,                  /* values for codes of length 4 */
      0x07,                  /* values for codes of length 5 */
      0x08,                  /* values for codes of length 6 */
      0x09,                  /* values for codes of length 7 */
      0x0a,                  /* values for codes of length 8 */
      0x0b,                  /* values for codes of length 9 */
      0x01,      /* table class 0, ID 1 */
      0x00, 0x03, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,   /* # codes of length 1..8 */
      0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,   /* # codes of length 9..16 */
      0x00, 0x01, 0x02,            /* values for codes of length 2 */
      0x03,                  /* values for codes of length 3 */
      0x04,                  /* values for codes of length 4 */
      0x05,                  /* values for codes of length 5 */
      0x06,                  /* values for codes of length 6 */
      0x07,                  /* values for codes of length 7 */
      0x08,                  /* values for codes of length 8 */
      0x09,                  /* values for codes of length 9 */
      0x0a,                  /* values for codes of length 10 */
      0x0b,                  /* values for codes of length 11 */
      0x10,
      0x00, 0x02, 0x01, 0x03, 0x03, 0x02, 0x04, 0x03,
      0x05, 0x05, 0x04, 0x04, 0x00, 0x00, 0x01, 0x7d,
      0x01, 0x02, 0x03, 0x00, 0x04, 0x11, 0x05, 0x12,
      0x21, 0x31, 0x41, 0x06, 0x13, 0x51, 0x61, 0x07,
      0x22, 0x71, 0x14, 0x32, 0x81, 0x91, 0xa1, 0x08,
      0x23, 0x42, 0xb1, 0xc1, 0x15, 0x52, 0xd1, 0xf0,
      0x24, 0x33, 0x62, 0x72, 0x82, 0x09, 0x0a, 0x16,
      0x17, 0x18, 0x19, 0x1a, 0x25, 0x26, 0x27, 0x28,
      0x29, 0x2a, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39,
      0x3a, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49,
      0x4a, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59,
      0x5a, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69,
      0x6a, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79,
      0x7a, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89,
      0x8a, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98,
      0x99, 0x9a, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7,
      0xa8, 0xa9, 0xaa, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6,
      0xb7, 0xb8, 0xb9, 0xba, 0xc2, 0xc3, 0xc4, 0xc5,
      0xc6, 0xc7, 0xc8, 0xc9, 0xca, 0xd2, 0xd3, 0xd4,
      0xd5, 0xd6, 0xd7, 0xd8, 0xd9, 0xda, 0xe1, 0xe2,
      0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea,
      0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8,
      0xf9, 0xfa, 0x11, 0x00, 0x02, 0x01, 0x02, 0x04,
      0x04, 0x03, 0x04, 0x07, 0x05, 0x04, 0x04, 0x00,
      0x01, 0x02, 0x77, 0x00, 0x01, 0x02, 0x03, 0x11,
      0x04, 0x05, 0x21, 0x31, 0x06, 0x12, 0x41, 0x51,
      0x07, 0x61, 0x71, 0x13, 0x22, 0x32, 0x81, 0x08,
      0x14, 0x42, 0x91, 0xa1, 0xb1, 0xc1, 0x09, 0x23,
      0x33, 0x52, 0xf0, 0x15, 0x62, 0x72, 0xd1, 0x0a,
      0x16, 0x24, 0x34, 0xe1, 0x25, 0xf1, 0x17, 0x18,
      0x19, 0x1a, 0x26, 0x27, 0x28, 0x29, 0x2a, 0x35,
      0x36, 0x37, 0x38, 0x39, 0x3a, 0x43, 0x44, 0x45,
      0x46, 0x47, 0x48, 0x49, 0x4a, 0x53, 0x54, 0x55,
      0x56, 0x57, 0x58, 0x59, 0x5a, 0x63, 0x64, 0x65,
      0x66, 0x67, 0x68, 0x69, 0x6a, 0x73, 0x74, 0x75,
      0x76, 0x77, 0x78, 0x79, 0x7a, 0x82, 0x83, 0x84,
      0x85, 0x86, 0x87, 0x88, 0x89, 0x8a, 0x92, 0x93,
      0x94, 0x95, 0x96, 0x97, 0x98, 0x99, 0x9a, 0xa2,
      0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa8, 0xa9, 0xaa,
      0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7, 0xb8, 0xb9,
      0xba, 0xc2, 0xc3, 0xc4, 0xc5, 0xc6, 0xc7, 0xc8,
      0xc9, 0xca, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd7,
      0xd8, 0xd9, 0xda, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6,
      0xe7, 0xe8, 0xe9, 0xea, 0xf2, 0xf3, 0xf4, 0xf5,
      0xf6, 0xf7, 0xf8, 0xf9, 0xfa
};

int zr36060_set_cap(struct zr36060_dev *zr, enum buz_codec_mode mode)
{
	u8	sof[19];
	u8	data[1024];
	u32	size;
	int	lines;
	int	i;
	int	len;
	int	Vstart;
	int	Hstart;
	struct	tvnorm *tvn;

	/* reset zr36060 on wake it up */
	zr->codec_wakeup(zr, 1);
 
	data[0] = 0x01;
	zr->codec_write_memory(zr, data, 0, 1);

	memset(data, 0, sizeof(data));

	data[0x02] = 0x00;	/* from matrox */

	if (mode == BUZ_MODE_MOTION_COMPRESS || mode == BUZ_MODE_STILL_COMPRESS) {
		data[0x03] = 0xa4; /* compress pass, variable scaling */
	} else {
		data[0x03] = 0x00; /* decompress pass */
	}
	data[0x04] = 0x00; /* reserved */
	data[0x05] = 0xff; /* max block code volume in bits (510) */
	data[0x06] = zr->params.jpeg_markers & 0x38;
	if (zr->params.COM_len) 
		data[0x06] |= JPEG_MARKER_COM;
	if (zr->params.APP_len)
		data[0x06] |= JPEG_MARKER_APP;

	data[0x07] = 0x01; /* from matrox */

	/* Target field size in pixels: */
	tvn = &zr->norms[zr->params.norm];
	size = (tvn->Ha/2)*(tvn->Wa)/(zr->params.HorDcm)/(zr->params.VerDcm);
         
	/* Target compressed field size in bits: */
	size = size*16; /* uncompressed size in bits */
	size = size*zr->params.quality/400; /* quality = 100 is a compression ratio 1:4 */

	if (size < 8192)
		size = 8192;

	/* Upper limit: 7/8 of the code buffers */
	if(size*zr->params.field_per_buff > zr->jpg_bufsize*7)
		size = zr->jpg_bufsize*7 / zr->params.field_per_buff;

	data[0x09] = (size >> 24) & 0xff;
	data[0x0a] = (size >> 16) & 0xff;
	data[0x0b] = (size >> 8) & 0xff;
	data[0x0c] = size & 0xff;

	data[0x11] = 0x01;	/* Scale Factor 1:1 */
	data[0x12] = 0x00;

	data[0x13] = 0xff;	/* location factor Must be 0xffffff */
	data[0x14] = 0xff;
	data[0x15] = 0xff;

	data[0x24] = 0x00;
	data[0x25] = 0x00;

	data[0x30] = 0x04;

	if (zr->video_bus_size == 8)
		data[0x30] = 0x80;

	if (zr->params.odd_even)
		data[0x30] |= (1 << 3);
	if (mode != BUZ_MODE_STILL_DECOMPRESS) {
		/* limit pixels to range 16..235 as per CICIR-601 */
		data[0x30] |= (1 << 6);
	}

	data[0x31] = 0x07;

	switch(zr->params.HorDcm) {
	default:
	case 1:
		data[0x32] = 0;
		break;
	case 2:
		data[0x32] = 1;
		break;

	case 4:
		data[0x32] = 2;
		break;
	}

	if (zr->params.VerDcm == 2)
		data[0x32] |= 0x04;

	data[0x33] = 0x80;
	data[0x34] = 0xe0;
	data[0x35] = 0xe0;
	data[0x36] = (tvn->Ht -1) >> 8;
	data[0x37] = (tvn->Ht -1);
	data[0x38] = (tvn->Wt -1) >> 8;
	data[0x39] = (tvn->Wt -1);
	data[0x3a] = 5;
	data[0x3b] = 99;
	switch(mode) {
	case BUZ_MODE_MOTION_DECOMPRESS:
	case BUZ_MODE_STILL_DECOMPRESS:
		Vstart = tvn->Vjpeg;
		Hstart = tvn->Hjpeg;
		break;

	case BUZ_MODE_MOTION_COMPRESS:
	case BUZ_MODE_STILL_COMPRESS:
	default:
		Vstart = tvn->VStart;
		Hstart = tvn->HStart;
		break;
	}

	data[0x3c] = tvn->VStart -1;
	data[0x3d] = Hstart -1;

	data[0x3e] = ((tvn->VStart-1) + tvn->Ha/2) >> 8;
	data[0x3f] = ((tvn->VStart-1) + tvn->Ha/2);

	data[0x40] = ((tvn->HStart-1) + tvn->Wa) >> 8;
	data[0x41] = ((tvn->HStart-1) + tvn->Wa);
	
	data[0x42] = (zr->params.img_y + Vstart) >> 8;
	data[0x43] = zr->params.img_y + Vstart;


	data[0x44] = ((zr->params.img_y + Vstart) + zr->params.img_height) >> 8;
	data[0x45] = ((zr->params.img_y + Vstart) + zr->params.img_height);

	data[0x46] = (zr->params.img_x + Hstart) >> 8;
	data[0x47] = (zr->params.img_x + Hstart);

	data[0x48] = ((zr->params.img_x + Hstart) + zr->params.img_width) >> 8;
	data[0x49] = ((zr->params.img_x + Hstart) + zr->params.img_width);


	data[0x4a] = (zr->params.img_y + Vstart) >> 8;
	data[0x4b] = zr->params.img_y + Vstart;

	data[0x4c] = ((zr->params.img_y + Vstart) + zr->params.img_height) >> 8;
	data[0x4d] = ((zr->params.img_y + Vstart) + zr->params.img_height);

	data[0x4e] = (zr->params.img_x + Hstart) >> 8;
	data[0x4f] = (zr->params.img_x + Hstart);

	data[0x50] = ((zr->params.img_x + Hstart) + zr->params.img_width) >> 8;
	data[0x51] = ((zr->params.img_x + Hstart) + zr->params.img_width);

	memcpy(&data[0x060], SOF, sizeof(sof));
	memcpy(&data[0x7a], SOS, sizeof(SOS));
	memcpy(&data[0xc0], DRI, sizeof(DRI));
	memcpy(&data[0xcc], DQT, sizeof(DQT));
	memcpy(&data[0x1d4], DHT, sizeof(DHT));

	lines = zr->params.img_height / zr->params.VerDcm;
	data[0x65] = lines >> 8;
	data[0x66] = lines;

	lines = zr->params.img_width / zr->params.HorDcm;
	data[0x67] = lines >> 8;
	data[0x68] = lines;

	len = zr->params.APP_len;
	if (len < 0) 
		len = 0;
	if (len > 60)
		len = 60;

	i = zr->params.APPn;
	if (i < 0) i = 0;
	if (i > 15) i= 15;

	data[0x380] = 0xff;
	data[0x381] = 0xe0 + i;
	data[0x382] = 0x00;
	data[0x383] = len + 2;
	for(i=0; i < len; i++)
		data[0x384+i] = zr->params.APP_data[i];


	len = zr->params.COM_len;
	if (len < 0) 
		len = 0;
	if (len > 60)
		len = 60;

	data[0x3c0] = 0xff;
	data[0x3c1] = 0xfe;
	data[0x3c2] = 0x00;
	data[0x3c3] = len + 2;
	for(i=0; i < len; i++)
		data[0x3c4+i] = zr->params.COM_data[i];

	if (zr->codec_write_memory(zr, &data[2], 0x02, 6))
		return -1;
	if (zr->codec_write_memory(zr, &data[9], 0x09, 4))
		return -1;
	if (zr->codec_write_memory(zr, &data[0x11], 0x11, 5))
		return -1;
	if (zr->codec_write_memory(zr, &data[0x24], 0x24, 2))
		return -1;
	if (zr->codec_write_memory(zr, &data[0x30], 0x30, 34))
		return -1;
	if (zr->codec_write_memory(zr, &data[0x60], 0x60, 19))
		return -1;
	if (zr->codec_write_memory(zr, &data[0x7a], 0x7a, 14))
		return -1;
	if (zr->codec_write_memory(zr, &data[0xc0], 0xc0, 6))
		return -1;
	if (zr->codec_write_memory(zr, &data[0xcc], 0xcc, 134))
		return -1;
	if (zr->codec_write_memory(zr, &data[0x1d4], 0x1d4, 420))
		return -1;
	if (zr->codec_write_memory(zr, &data[0x380], 0x380, 64))
		return -1;
	if (zr->codec_write_memory(zr, &data[0x3c0], 0x3c0, 64))
		return -1;

	return zr36060_load_memory(zr);
}

int
zr36060_interrupt(struct zr36060_dev *zr36060)
{
	printk("zr36060: command complete interrupt\n");
#ifdef	NEVER
	zr->state = ZR_IDLE_STATE;

	/* remove pending command complete */
	writeb(VI_CMDCMPL, zr->viclear);
#endif
	return 0;
}

static void jpg_enable(struct zr36060_dev *zr, enum buz_codec_mode mode)
{
   int        	on;

#ifdef	NEVER
   on = zr->v4l_overlay_active;
#else
  on = 0;
#endif

   if (on && zr->codec_overlay) 
	zr->codec_overlay(zr, 0);

   switch (mode) {

   case BUZ_MODE_MOTION_COMPRESS:
      zr36060_set_cap(zr, mode);
      zr->codec_set_jpg(zr, mode);
      zr->codec_set_video(zr, mode);
      zr->codec_transfer(zr, mode);
      break;

   case BUZ_MODE_MOTION_DECOMPRESS:
      zr->codec_set_video(zr, mode);
      zr36060_set_cap(zr, mode);
      zr->codec_set_jpg(zr, mode);
      zr->codec_transfer(zr, mode);
      break;

   case BUZ_MODE_IDLE:
   default:
      /* shut down processing */
      zr->codec_transfer(zr, mode);
      zr->codec_wakeup(zr, 0);
      zr->codec_set_video(zr, mode);
      break;
   }

   zr->codec_mode = mode;

   if (on && zr->codec_overlay) 
	zr->codec_overlay(zr, 1);
}

/*
 *   Queue a MJPEG buffer for capture/playback
 */

static int jpg_qbuf(struct zr36060_dev *zr, int frame, enum buz_codec_mode mode)
{
   unsigned long flags;
   int res;

   /* Check if buffers are allocated */

   if(!zr->jpg_buffers_allocated)
   {
      printk(ZR_ERR "-%u: jpg_qbuf: buffers not yet allocated\n", zr->id);
      return -ENOMEM;
   }

   /* Does the user want to stop streaming? */

   if (frame < 0) {
      if (zr->codec_mode==mode) {
         jpg_enable(zr, BUZ_MODE_IDLE);
         return 0;
      }
      else
      {
         printk(ZR_ERR  "-%u: jpg_qbuf - stop streaming but not in streaming mode\n", zr->id);
         return -EINVAL;
      }
   }

   /* No grabbing outside the buffer range! */

   if (frame >= zr->jpg_nbufs) {
      printk(ZR_ERR "-%u: jpg_qbuf: buffer %d out of range\n",frame, zr->id);
      return -EINVAL;
   }

   /* what is the codec mode right now? */
#ifdef	NEVER
   if ( btv->codec_mode == BUZ_MODE_IDLE )
   {
      /* Ok load up the zr36060 and go */
      jpg_enable(btv, mode);
   }
   else 
if ( btv->codec_mode != mode )
   {
      /* wrong codec mode active - invalid */
      printk(ZR_ERR ": jpg_qbuf - codec in wrong mode\n");
      return -EINVAL;
   }
#endif

   spin_lock_irqsave(&zr->lock, flags);

   /* make sure a grab isn't going on currently with this buffer */
   switch (zr->jpg_gbuf[frame].state) {

   default:
   case BUZ_STATE_DMA:
   case BUZ_STATE_PEND:
   case BUZ_STATE_DONE:
      res = -EBUSY;         /* what are you doing? */
      break;

   case BUZ_STATE_USER:
      /* since there is at least one unused buffer there's room for at least one more pend[] entry */
      zr->jpg_pend[zr->jpg_que_head++ & BUZ_MASK_FRAME] = frame;
      zr->jpg_gbuf[frame].state = BUZ_STATE_PEND;
printk("jpg_que_head = %d\n", zr->jpg_que_head);
#ifdef	NEVER
      buz_feed_stat_com(btv);
#endif
      res = 0;
      break;

   }

   spin_unlock_irqrestore(&zr->lock, flags);

   if ( zr->codec_mode == BUZ_MODE_IDLE )
   {
      /* Ok load up the zr36060 and go */
      jpg_enable(zr, mode);
   }

   /* Start the zr36060 when the first frame is queued  */
   if(zr->jpg_que_head==1)
   {
#ifdef	NEVER
      btor(ZR36057_JMC_Go_en, ZR36057_JMC);
      btwrite(ZR36057_JPC_P_Reset | ZR36057_JPC_CodTrnsEn | ZR36057_JPC_Active, ZR36057_JPC);
#endif
   }

   return res;
}

/*
 *   Sync on a MJPEG buffer
 */

static int jpg_sync(struct zr36060_dev *zr, struct buz_sync * bs)
{
   unsigned long flags;
   int frame;

   if( zr->codec_mode != BUZ_MODE_MOTION_DECOMPRESS &&
       zr->codec_mode != BUZ_MODE_MOTION_COMPRESS ) {
      printk(ZR_ERR "-%u: BUZIOCSYNC: - codec not in streaming mode\n", zr->id);
      return -EINVAL;
   }

   while(zr->jpg_que_tail == zr->jpg_dma_tail) {
      interruptible_sleep_on(&zr->jpg_capq);
      if (signal_pending(current)) return -ERESTARTSYS;
   }

   spin_lock_irqsave(&zr->lock, flags);

   frame = zr->jpg_pend[zr->jpg_que_tail++ & BUZ_MASK_FRAME];
printk("sync frame buffer %d\n", frame);

   /* buffer should now be in BUZ_STATE_DONE */

   if(zr->jpg_gbuf[frame].state != BUZ_STATE_DONE)
      printk(ZR_ERR "-%u: jpg_sync - internal error\n", zr->id);

   *bs = zr->jpg_gbuf[frame].bs;
   zr->jpg_gbuf[frame].state = BUZ_STATE_USER;

   spin_unlock_irqrestore(&zr->lock, flags);

   return 0;
}
int
zr36060_done(struct zr36060_dev *zr)
{
	int	frame;
	struct	buz_gbuffer *gbuf;

	if (zr->jpg_dma_tail== zr->jpg_que_head) {
		return 0;
	}

	frame = zr->jpg_pend[zr->jpg_dma_tail & BUZ_MASK_FRAME];
	gbuf = &zr->jpg_gbuf[frame];
	get_fast_time(&gbuf->bs.timestamp);

	gbuf->state = BUZ_STATE_DONE;
	gbuf->bs.length = 0;

	zr->jpg_dma_tail++;

	wake_up_interruptible(&zr->jpg_capq);

	return 0;
}

int
zr36060_get_field(struct zr36060_dev *zr, void *start, int size, void *start1, int size1)
{
	int		frame;
	int		frame_size;
	int		state;
	int		i;
	int		count;
	unsigned char	*addr;

	struct	buz_gbuffer *gbuf;

	if (zr->jpg_dma_head == zr->jpg_que_head) {
		return 0;
	}

	frame = zr->jpg_pend[zr->jpg_dma_tail & BUZ_MASK_FRAME];
	gbuf = &zr->jpg_gbuf[frame];

	if (gbuf->bs.length == 0) {
		i = 0;
		addr = (void *)gbuf->virt_tab[0];
		count = gbuf->virt_tab[1];
		frame_size = 0;
		state = 0;
		i += 2;
		for(;;) {

			frame_size++;

			switch(state) {

			case 0:
				if (addr[0] == 0xff)
					state = 1;
				break;

			case 1:
				if (addr[0] == 0xd9)
					state = 2;
				else if (addr[0] == 0xff)
					state = 1;
				else 
					state = 0;
				break;

			case 2:
				break;
	
			}

			if (state == 2) {
				gbuf->bs.length = frame_size;
				break;
			}

			count--;
			addr++;

			if (count <= 0) {
				addr = (void *)gbuf->virt_tab[i];
				count = gbuf->virt_tab[i+1];
				if (gbuf->frag_tab[i+1] & 0x1) {
					printk("Bad buffer no end of image\n");
					return 0;
				}
				i += 2;
			}
		}
	}

	
	if (gbuf->bs.length > size + size1)
		return 0;

	gbuf->state = BUZ_STATE_DMA;
	zr->jpg_dma_head++;

	if (gbuf->bs.length < size) {
		size = gbuf->bs.length;
	}

	size1 = gbuf->bs.length - size;

	i = 0;
	if (start && size) {
		count = 0;
		addr = NULL;

		while(size > 0) {
			addr = (void *)gbuf->virt_tab[i];
			count = gbuf->virt_tab[i+1];
			if (count > size)
				count = size;
			memcpy(start, addr, count);
			size -= count;
			start += count;
			i += 2;
		}

		if (start1 && size1) {
			addr += count;
			if (count > size1)
				count = size1;
			memcpy(addr, start1, count);
			size1 -= count;
			start1 += count;

			while(size1 > 0) {
				addr = (void *)gbuf->virt_tab[i];
				count = gbuf->virt_tab[i+1];
				if (count > size1)
					count = size1;
				memcpy(start1, addr, count);
				size1 -= count;
				start1 += count;
				i += 2;
			}
		}
	}
	return gbuf->bs.length;
}

int
zr36060_new_field(struct zr36060_dev *zr, void *start, int size, void *start1, int size1)
{
	int	frame;
	int	i;
	int	count;
	void	*addr;

	struct	buz_gbuffer *gbuf;

	if (zr->jpg_dma_head == zr->jpg_que_head) {
		return 0;
	}

	if (size > zr->jpg_bufsize) {
		printk("image to big for buffer\n");
		size = zr->jpg_bufsize;
		size1 = 0;
	}

	if (size + size1 >zr->jpg_bufsize) {
		printk("image to big for buffer\n");
		size1 = zr->jpg_bufsize - size;
	}

	frame = zr->jpg_pend[zr->jpg_dma_head++ & BUZ_MASK_FRAME];
	gbuf = &zr->jpg_gbuf[frame];

	gbuf->bs.length = size + size1;	
	gbuf->bs.seq = zr->jpg_seq_num++;
	get_fast_time(&gbuf->bs.timestamp);

	gbuf->state = BUZ_STATE_DONE;
	zr->jpg_dma_tail++;

	if (start) {
		i = 0;
		count = 0;
		addr = NULL;

		while(size > 0) {
			addr = (void *)gbuf->virt_tab[i];
			count = gbuf->virt_tab[i+1];
			if (count > size)
				count = size;
			memcpy(addr, start, count);
			size -= count;
			start += count;
			i += 2;
		}

		if (start1) {
			addr += count;
			if (count > size1)
				count = size1;
			memcpy(addr, start1, count);
			size1 -= count;
			start1 += count;

			while(size1 > 0) {
				addr = (void *)gbuf->virt_tab[i];
				count = gbuf->virt_tab[i+1];
				if (count > size1)
					count = size1;
				memcpy(addr, start1, count);
				size1 -= count;
				start1 += count;
				i += 2;
			}
		}
	}

	wake_up_interruptible(&zr->jpg_capq);

	return 0;
}


int zr36060_ioctl(struct zr36060_dev *zr, int command, void *arg)
{
	switch(command) {

	case ZR36060_DEF_PARAMS:
		return zr36060_default_params(zr);

	case ZR36060_GET_PARAMS:
		memcpy(arg, &zr->params, sizeof(zr->params));
		return 0;

	case ZR36060_SET_PARAMS:
		if (zr->codec_mode != BUZ_MODE_IDLE) {
			printk(ZR_ERR "%-u: SET_PARAMS called but zr36060 in capture/play mode\n", 0);
			return -EINVAL;
		}

		if (zr36060_check_params(zr, (struct buz_params *)arg))
			return EINVAL;

		zr->params = *(struct buz_params *)arg;
		return 0;

	case ZR36060_GET_MODE:
		*(enum buz_codec_mode *)arg = zr->codec_mode;
		return 0;

	case ZR36060_SET_MODE:
		jpg_enable(zr, *(enum buz_codec_mode *)arg);
		return 0;

	case ZR36060_REQBUFS:
	{
		struct buz_requestbuffers *br;

		br = (struct buz_requestbuffers *)arg;

		if(zr->jpg_buffers_allocated) {
			printk(ZR_ERR "-%u: buffers already allocated\n", zr->id);
			return -EINVAL;
		}

		/* Enforce reasonable lower and upper limits */
		if(br->count<4)
			br->count = 4; /* Could be chosen smaller */

		if(br->count>BUZ_MAX_FRAME) 
			br->count = BUZ_MAX_FRAME;

		br->size = PAGE_ALIGN(br->size);

		if(br->size < 8192)
			br->size = 8192; /* Arbitrary */

		/* br->size is limited by 1 page for the stat_com 
		   tables to a Maximum of 2 MB */
		if(br->size > (512*1024)) 
			br->size = (512*1024); /* 512 K should be enough */

			
		if(zr->need_contiguous && br->size>MAX_KMALLOC_MEM) 
			br->size = MAX_KMALLOC_MEM;

		zr->jpg_nbufs   = br->count;
		zr->jpg_bufsize = br->size;

		if(jpg_fbuffer_alloc(zr)) 
			return -ENOMEM;

		/* The next mmap will map the MJPEG buffers */
		zr->map_mjpeg_buffers = 1;

		return 0;
	}

	case ZR36060_CANBUFS:
		jpg_fbuffer_free(zr);
		zr->jpg_nbufs = 0;
		return 0;

	case ZR36060_SYNC:
		return jpg_sync(zr, (struct buz_sync *)arg);

	case ZR36060_QBUF_CAPT:
		return jpg_qbuf(zr, *(int *)arg, BUZ_MODE_MOTION_COMPRESS);

	case ZR36060_QBUF_PLAY:
		return jpg_qbuf(zr, *(int *)arg, BUZ_MODE_MOTION_DECOMPRESS);

	default:
		return -1;
	}
	return -1;
}


/* This is run from mgavideo.c - it attaches this module to the "public" 
   communication paths */
int     zr36060_attach(struct zr36060_dev *zr)
{
	char	chip_id[2];
	int	addr;
	int	j;

	if (debug)
		printk("zr36060: attaching zr36060\n");

	zr->initialized = 0;
	zr->user = 0;

	zr->map_mjpeg_buffers = 0; /* Map V4L buffers by default */

	zr->jpg_nbufs   = 0;
	zr->jpg_bufsize = 0;
	zr->jpg_buffers_allocated = 0;

#ifdef	NEVER
	zr->params.norm     = default_norm  ? 1 : 0; /* Avoid nonsense settings from user */
	zr->params.input    = default_input ? 1 : 0; /* Avoid nonsense settings from user */
#else
	zr->params.norm     = 0;
	zr->params.input    = 0;
#endif

	for (j = 0; j < BUZ_MAX_FRAME; j++) {
		zr->jpg_gbuf[j].virt_tab       = 0;
		zr->jpg_gbuf[j].frag_tab       = 0;
		zr->jpg_gbuf[j].frag_tab_bus   = 0;
		zr->jpg_gbuf[j].state          = BUZ_STATE_USER;
		zr->jpg_gbuf[j].bs.frame       = j;
	}



	/* initialize the shadow register to invalid values */
	for (addr=0; addr <= 0xff; addr++)
	  zr->shadow_register[addr] = 0xabcd;

	
	zr->codec_wakeup(zr, 1);


	/* read id from zr36060 */
	if (debug)
	  printk("zr36060: reading chip id\n");


	if (zr->codec_read_memory(zr, chip_id, 0x22, 2)) {
		printk("zr36060: Read error\n");
		return -1;
		}

	if (chip_id[0] == 0x33) {
		printk("zr36060: zr36060 found revision %x\n", chip_id[1]);
	} else {
		printk("zr36060: zr36060 not found (0x%x 0x%x)\n",
		       chip_id[0], chip_id[1]);
		return -1;
	}

	zr36060_default_params(zr);

	zr->initialized = 1;

	zr->codec_wakeup(zr, 0);

#ifdef	NEVER
	MOD_INC_USE_COUNT;
#endif
	return 0;
}

int
zr36060_detach(struct zr36060_dev *zr36060)
{
#ifdef	NEVER
	/* reset the codec interface */
	writeb(0x00, zr->codecctl);
	writeb(0x01, zr->codecctl);
	writeb(0x00, zr->codecctl);

	MOD_DEC_USE_COUNT;
#endif
	return 0;
}



